/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * <summary> StatsResult is container for multiple results, each of which are mapped from a Metric to a Long. This
 * class wraps some map operations as well as provides a way to clone and aggregate results. </summary>
 *
 * <thread-safety> This class is not thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsResult {

    /**
     * <summary> The results mapping from metric to the statistical quantity (Long). This reference is never
     * changed. </summary>
     */
    private final Map results = new HashMap();

    /**
     * <summary> Default constructor. Does nothing. </summary>
     */
    public StatsResult() {
        // empty
    }

    /**
     * <summary> Create a result based on another result. This acts as a way to clone the instance. </summary>
     *
     * @param result the result to clone from
     * @throws NullPointerException if result is null
     */
    public StatsResult(StatsResult result) {
        CheckHelper.checkNull(result, "result");
        this.results.putAll(result.results);
    }

    /**
     * <summary> Set a certain stat (associate metric with value). This method will overwrite old values
     * associated. </summary>
     *
     * @param metric the metric to associate with
     * @param result the quantity
     *
     * @throws NullPointerException if metric is null
     */
    public void setStat(Metric metric, long result) {
        CheckHelper.checkNull(metric, "metric");
        results.put(metric, new Long(result));
    }

    /**
     * <summary> Get the quantity associated with the metric. </summary>
     *
     * @return the quantity
     * @param metric the metric associated with value
     *
     * @throws NullPointerException if metric is null
     * @throws IllegalArgumentException if no value associated
     */
    public long getStat(Metric metric) {
        CheckHelper.checkNull(metric, "metric");
        Object value = results.get(metric);
        if (value == null) {
            throw new IllegalArgumentException("No value associated with metric: " + metric.getName());
        }
        return ((Long) value).longValue();
    }

    /**
     * <summary> Get a list of all the associated metrics. Should return a copy from inner collection. </summary>
     *
     * @return a list of associated metrics
     */
    public List getMetrics() {
        return new ArrayList(results.keySet());
    }

    /**
     * <summary> Aggregate the result into the current result. </summary>
     *
     * @param result the result to aggregate from
     * @throws NullPointerException if result is null
     */
    public void aggregate(StatsResult result) {
        CheckHelper.checkNull(result, "result");

        // for each entry from given result, add its value to current results
        for (Iterator iter = result.results.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            Metric key = (Metric) entry.getKey();
            long value = ((Long) entry.getValue()).longValue();

            // add the value to corresponding current one, or put new entry if not exists
            Long myValue = (Long) this.results.get(key);
            if (myValue == null) {
                this.results.put(key, entry.getValue());
            } else {
                this.results.put(key, new Long(value + myValue.longValue()));
            }
        }
    }
}
