/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.matcher;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import com.topcoder.file.statistics.FileMatcher;

/**
 * <summary> FileCategoryMatcher is a template class that handles file category (text/binary) oriented matching
 * logic. The purpose to still create subclasses for such minor functionality is about the convenience in
 * configuration. </summary>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileCategoryMatcher implements FileMatcher {

    /** The number of bytes to check at most. */
    private static final int BYTES_TO_CHECK = 1024;

    /** The upper bound for character code allowed in a text file. */
    private static final int MAX_CHARCODE = 127;

    /**
     * <summary> Default constructor. Does nothing. </summary>
     */
    protected FileCategoryMatcher() {
        // empty
    }

    /**
     * <summary> It checks if the file is binary. This method is exposed as static public because it could
     * potentially be used by other custom matchers. </summary>
     *
     * <algorithm> 1) Open a binary stream and check at most 1024 bytes; 2) If a byte is not within [32, 127] and
     * is not tab, carriage return or new line, it's a binary file; 3) Otherwise it's a text file. 4) In any case
     * the stream should be closed. </algorithm>
     *
     * @return whether the file is binary
     * @param file the file to check
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public static boolean isBinary(File file) throws IOException {
        CheckHelper.checkNull(file, "file");
        InputStream stream = new BufferedInputStream(new FileInputStream(file));
        try {
            for (int i = 0; i < BYTES_TO_CHECK; i++) {
                int b = stream.read();
                if (b == -1) {
                    break;
                }
                if ((b < ' ' || b > MAX_CHARCODE) && !(b == '\t' || b == '\r' || b == '\n')) {
                    return true;
                }
            }
            return false;
        } finally {
            stream.close();
        }
    }

    /**
     * <summary> It uses isBinary() to check if the file is binary, and passes the flag to the template method to
     * decide. </summary>
     *
     * @return whether the file matches
     * @param file the file to match
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public boolean match(File file) throws IOException {
        return match(isBinary(file));
    }

    /**
     * <summary> Decide whether the file category matches. </summary>
     *
     * @return whether the category matches
     * @param isBinary whether the file is binary
     */
    protected abstract boolean match(boolean isBinary);
}
