/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;
import com.topcoder.util.config.Property;

/**
 * <summary> TextLineCounter counts lines for text files. It supports a single metric of "Line Count". </summary>
 *
 * <configuration> It delegates configuration to base class without any addition. </configuration>
 *
 * <thread-safety> This class is stateless while its base class is thread-safe (thus it is thread-safe too).
 * </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class TextLineCounter extends FileProcessorBase {

    /**
     * <summary> The name of the metric used for this processor. </summary>
     */
    public static final String NAME = "Line Count";

    /**
     * <summary> The display name of the metric used for this processor. </summary>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <summary> The metric used by this processor. </summary>
     */
    private static final Metric METRIC = new Metric(NAME, DISPLAY_NAME, "Number of lines in a text file.");

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * @param alias the alias
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public TextLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * @param property the property to load config from
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public TextLineCounter(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <summary> Return a list contains single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </summary>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        return Collections.singletonList(METRIC);
    }

    /**
     * <summary> Count the lines in text file. </summary>
     *
     * <algorithm> 1) Create a text stream. 2) Count the lines until EOF is met. 3) Close the stream. 4) Create a
     * StatsResult and place the line count under the specified metric. </algorithm>
     *
     * @return the line count result
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public StatsResult process(File file) throws IOException {
        CheckHelper.checkNull(file, "file");

        // read text file line by line and count the lines
        long lines = 0;
        BufferedReader reader = new BufferedReader(new FileReader(file));
        try {
            for (;;) {
                String line = reader.readLine();
                if (line == null) {
                    break;
                }
                lines++;
            }
        } finally {
            reader.close();
        }

        // return number of lines as a stats result
        StatsResult result = new StatsResult();
        result.setStat(METRIC, lines);
        return result;
    }
}
