/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.reporting;

import java.io.File;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.util.config.Property;

/**
 * <summary> This class provides a StatsReporting implementation base. It aggregates results from each processor
 * and classifies them based on single file, directory or alias. It also provides an overall stats. It defers the
 * actual rendering logic to concrete implementations (BasicStatsReporting and XmlStatsReporting). </summary>
 *
 * <configuration> This class has a constructor with Property for extension purpose. There is no real logic in this
 * release. </configuration>
 *
 * <thread-safety> This class is not thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractStatsReporting implements StatsReporting {

    /**
     * <summary> The stats mapped to each file. The file path is relative to base dir or absolute if no base dir.
     * Its reference never changes. </summary>
     */
    private final StatsCollection fileStats = new StatsCollection();

    /**
     * <summary> The stats mapped to each directory. The dir path is relative to base dir or absolute if no base
     * dir. Its reference never changes. </summary>
     */
    private final StatsCollection directoryStats = new StatsCollection();

    /**
     * <summary> The stats mapped to each alias. </summary>
     */
    private StatsCollection aliasStats = new StatsCollection();

    /**
     * <summary> The overall stats. It is created anew when reporting resets. </summary>
     */
    private StatsResult overallStats = new StatsResult();

    /**
     * <summary> The base dir for the reporting. This variable is nullable (meaning all paths are absolute).
     * </summary>
     */
    private File baseDirectory = null;

    /**
     * <summary> Default constructor. Does nothing. </summary>
     */
    protected AbstractStatsReporting() {
        // empty
    }

    /**
     * <summary> Constructor with property. This does not have any functionality in the current release and is kept
     * for extension purpose. The configuration exception must be declared for extenstion purpose.</summary>
     *
     * @param property the property to load config from
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    protected AbstractStatsReporting(Property property) throws ConfigurationException {
        CheckHelper.checkNull(property, "property");
    }

    /**
     * <summary> Aggregate a result into the reporting. </summary>
     *
     * <algorithm> 1) Get the file name. If base dir exists, strip the base portion off. 2) Aggregate the result
     * against file, each path it is in, alias and the overall stats. 3) Notice the dir stats does not need to
     * include the base dir (since it's same as overall). </algorithm>
     *
     * @param processor the processor that generated the result
     * @param file the file that is processed
     * @param result the result
     *
     * @throws NullPointerException if processor or file or result is null
     * @throws StatsReportingException if file is not in base dir
     */
    public void aggregate(FileProcessor processor, File file, StatsResult result) throws StatsReportingException {
        CheckHelper.checkNull(processor, "processor");
        CheckHelper.checkNull(file, "file");
        CheckHelper.checkNull(result, "result");

        // strip base dir from file path
        if (baseDirectory != null) {
            String baseDir = baseDirectory.getPath();
            String filePath = file.getPath();
            if (!filePath.startsWith(baseDir)) {
                throw new StatsReportingException("File path is outside base directory, file: " + file
                        + " baseDir:" + baseDirectory);
            }
            file = new File(filePath.substring(baseDir.length() + 1));
        }

        // update file stats
        fileStats.add(file.getPath(), result);

        // update directory stats
        for (;;) {
            file = file.getParentFile();
            // if there is no parent file, or we reached base directory, exit loop
            if (file == null || (baseDirectory != null && file.getPath().equals(baseDirectory.getPath()))) {
                break;
            }
            directoryStats.add(file.getPath(), result);
        }

        // update alias and overall stats
        aliasStats.add(processor.getAlias(), result);
        overallStats.aggregate(result);
    }

    /**
     * <summary> Reset the reporting. Base dir can be null when absolute paths are desirable. </summary>
     *
     * <algorithm> 1) Clear all the collections. 2) Create a new overall StatsResult. 3) Set the base dir.
     * </algorithm>
     *
     * @param baseDirectory the base dir to use
     *
     * @throws IllegalArgumentException if baseDirectory is not null and not a valid dir
     */
    public void reset(File baseDirectory) {
        // set the base directory
        if (baseDirectory != null && !baseDirectory.isDirectory()) {
            throw new IllegalArgumentException("Not a valid directory: " + baseDirectory);
        }
        this.baseDirectory = baseDirectory;
        // clear stats
        this.aliasStats.clear();
        this.directoryStats.clear();
        this.fileStats.clear();
        this.overallStats = new StatsResult();
    }

    /**
     * <summary> Get the file stats. Simply return the variable. </summary>
     *
     * @return the file stats
     */
    public StatsCollection getFileStats() {
        return fileStats;
    }

    /**
     * <summary> Get the dir stats. Simply return the variable. </summary>
     *
     * @return the dir stats
     */
    public StatsCollection getDirectoryStats() {
        return directoryStats;
    }

    /**
     * <summary> Get the alias stats. Simply return the variable. </summary>
     *
     * @return the alias stats
     */
    public StatsCollection getAliasStats() {
        return aliasStats;
    }

    /**
     * <summary> Get the overall stats. Return a clone. </summary>
     *
     * @return the overall stats
     */
    public StatsResult getOverallStats() {
        return new StatsResult(overallStats);
    }

    /**
     * <summary> Get the base dir. Simply return the variable. </summary>
     *
     * @return the base dir
     */
    public File getBaseDirectory() {
        return baseDirectory;
    }
}
