/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.reporting;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.topcoder.file.statistics.StatsResult;

/**
 * <summary> StatsCollection is a generic container for mapping from String keys to StatsResult instances. It is
 * used in AbstractStatsReporting to hold file, directory and alias classified mappings. The class provides basic
 * manipulation methods for the mapping. </summary>
 *
 * <thread-safety> This class is not thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsCollection {

    /**
     * <summary> Mapping from String to StatsResult. This is used to classify file, dir and alias basd stats in
     * AbstractStatsReporting. The reference never change. </summary>
     */
    private final Map stats = new HashMap();

    /**
     * <summary> Default constructor. Does nothing. </summary>
     */
    public StatsCollection() {
        // your code here
    }

    /**
     * <summary> Add a result to specified key. </summary>
     *
     * <algorithm> If mapping already exists, should aggregate the result with this one. Otherwise save a clone to
     * the mapping. </algorithm>
     *
     * @param key the key to associate result with
     * @param result the result to add
     *
     * @throws NullPointerException if key or result is null
     * @throws IllegalArgumentException if key is empty after trimmed
     */
    public void add(String key, StatsResult result) {
        CheckHelper.checkString(key, "key");
        CheckHelper.checkNull(result, "result");

        // if entry exists, aggregate the result with this one; otherwise, put a clone of to the stats mapping
        if (stats.containsKey(key)) {
            StatsResult current = (StatsResult) stats.get(key);
            current.aggregate(result);
        } else {
            stats.put(key, new StatsResult(result));
        }
    }

    /**
     * <summary> Get the stats result associated with a key. The result is cloned before return. If no result is
     * associated return null. </summary>
     *
     * @return the result associated with key
     * @param key the key to retrieve
     *
     * @throws NullPointerException if key is null
     * @throws IllegalArgumentException if key is empty after trimmed
     */
    public StatsResult get(String key) {
        CheckHelper.checkString(key, "key");
        StatsResult result = (StatsResult) stats.get(key);
        return result == null ? null : new StatsResult(result);
    }

    /**
     * <summary> Get the key set of the collection. Should return a copy and sort the strings in alphabetical order
     * (case insensitive). </summary>
     *
     * @return the key set
     */
    public List getAllKeys() {
        ArrayList list = new ArrayList(stats.keySet());
        Collections.sort(list, new Comparator() {

            /**
             * <summary>Compares two strings, case insensitive.</summary>
             *
             * @param o1 first string to compare
             * @param o2 second string to compare
             */
            public int compare(Object o1, Object o2) {
                String s1 = (String) o1;
                String s2 = (String) o2;
                return s1.compareToIgnoreCase(s2);
            }
        });

        return list;
    }

    /**
     * <summary> Clear the collection. </summary>
     */
    public void clear() {
        stats.clear();
    }
}
