/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.reporting;

import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.util.config.Property;

/**
 * <summary> XmlStatsReporting is a reporting implementation that renders XML reports. Refer to component
 * specification for formatting of the XML. It is capable of using a stylesheet to transform the XML into
 * user-oriented format (HTML, CSV, etc.) </summary>
 *
 * <configuration> It supports a single property "stylesheet" which contains a filename of the stylesheet. This
 * property is optional. If it's missing the raw XML is generated. </configuration>
 *
 * <thread-safety> This class is not thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReporting extends AbstractStatsReporting {

    /**
     * <summary> The property which specifies a stylesheet to apply on the xml. If this property is not specified,
     * the raw output is generated. </summary>
     */
    public static final String STYLESHEET_PROPERTY_KEY = "stylesheet";

    /**
     * <summary>A default transform that is used to serialize the prepared DOM document with report. It should
     * support pretty xml intendation, however, this depend on underlying xslt processor. This transform is written
     * to work with Xalan</summary>
     */
    private static final String IDENTITY_TRANSFORM = ""
            + "<xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'"
            + " xmlns:xalan='http://xml.apache.org/xalan'>"
            + "<xsl:output method='xml' indent='yes' xalan:indent-amount='4'/>"
            + "<xsl:template match='/'><xsl:copy-of select='*'/></xsl:template></xsl:stylesheet>";

    /**
     * <summary> The stylesheet to apply on the raw xml generated. This is initialized in constructor and never
     * changed. The value can be null indicating no stylesheet is used. </summary>
     */
    private final File stylesheet;

    /**
     * <summary> The field to cache the tranformer instance loaded from stylesheet. </summary>
     */
    private Transformer transformer;

    /**
     * <summary> Default constructor. </summary>
     */
    public XmlStatsReporting() {
        this.stylesheet = null;
    }

    /**
     * <summary> Constructor with a stylesheet. Simply store it to member variable. </summary>
     *
     * @param stylesheet the stylesheet
     *
     * @throws NullPointerException if stylesheet is null
     * @throws IllegalArgumentException if stylesheet is not a valid file
     */
    public XmlStatsReporting(File stylesheet) {
        CheckHelper.checkNull(stylesheet, "stylesheet");
        if (!stylesheet.isFile()) {
            throw new IllegalArgumentException("Not a valid file: " + stylesheet);
        }
        this.stylesheet = stylesheet;
    }

    /**
     * <summary> Configures from a property. Propagate it to super constructor. After that, it reads a String
     * property from "stylesheet" and store it to variable. This property is optional, but should be an existing
     * filename if exists. </summary>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public XmlStatsReporting(Property property) throws ConfigurationException {
        super(property);

        String value = property.getValue(STYLESHEET_PROPERTY_KEY);
        if (value != null) {
            stylesheet = new File(value);
            if (!stylesheet.isFile() || !stylesheet.exists()) {
                throw new ConfigurationException("Not a valid file or file not exists: " + stylesheet);
            }
        } else {
            stylesheet = null;
        }
    }

    /**
     * <summary> This generates an XML format report if no stylesheet is specified, or outputs the result of XSLT.
     * Refer to CS for details. </summary>
     *
     * @return the generated report
     *
     * @throws StatsReportingException if rendering fails
     */
    public String generateReport() throws StatsReportingException {

        // create DOM Document
        Document document;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.newDocument();
        } catch (ParserConfigurationException e) {
            throw new StatsReportingException("Failed to create xml document", e);
        }

        // generate xml into the document
        generateXmlReport(document);

        // transform xml with the stylesheet
        try {
            StringWriter sw = new StringWriter();
            createTransformer().transform(new DOMSource(document), new StreamResult(sw));
            return sw.toString();

        } catch (TransformerConfigurationException e) {
            throw new StatsReportingException("Failed to create xml transformer", e);
        } catch (TransformerException e) {
            throw new StatsReportingException("Failed to transform generated report xml document", e);
        }
    }

    /**
     * <summary>Obtains Transformer instance loaded from stylesheet. The transformed is cached in a field of this
     * object.</summary>
     *
     * @return Transformed instance for the stylesheet
     *
     * @throws TransformerConfigurationException if failed to load stylesheet
     */
    private Transformer createTransformer() throws TransformerConfigurationException {
        if (transformer == null) {
            TransformerFactory factory = TransformerFactory.newInstance();
            if (stylesheet != null) {
                transformer = factory.newTransformer(new StreamSource(stylesheet));
            } else {
                transformer = factory.newTransformer(new StreamSource(new StringReader(IDENTITY_TRANSFORM)));
            }
        }
        return transformer;
    }

    /**
     * <summary> Generates an xml report and fills in the given document. </summary>
     *
     * @param doc Xml DOM Document to fill the report into.
     */
    private void generateXmlReport(Document doc) {
        Element report = doc.createElement("report");
        doc.appendChild(report);

        // create and populate <file> element
        Element files = doc.createElement("files");
        report.appendChild(files);
        storeStatsCollectionAsXml(doc, files, getFileStats(), "file");

        // create and populate <directories> element
        Element directories = doc.createElement("directories");
        report.appendChild(directories);
        storeStatsCollectionAsXml(doc, directories, getDirectoryStats(), "directory");

        // create and populate <aliases> element
        Element aliases = doc.createElement("aliases");
        report.appendChild(aliases);
        storeStatsCollectionAsXml(doc, aliases, getAliasStats(), "alias");

        // create and populate <overall> element
        Element overall = doc.createElement("overall");
        report.appendChild(overall);
        storeStatsAsXml(doc, overall, getOverallStats());
    }

    /**
     * <summary>Fills given parent element with stats collection items with supplied element name. </summary>
     *
     * @param doc the owner document of the element
     * @param parent Element to populate with stats elements
     * @param stats StatsCollection with data to populate into the element
     * @param elemName String with element name to use for children of parent element
     */
    private void storeStatsCollectionAsXml(Document doc, Element parent, StatsCollection stats, String elemName) {
        List keys = stats.getAllKeys();
        // for each entry in stats collection, create an element
        for (Iterator iter = keys.iterator(); iter.hasNext();) {
            String key = (String) iter.next();
            Element elem = doc.createElement(elemName);
            elem.setAttribute("name", key);
            parent.appendChild(elem);
            storeStatsAsXml(doc, elem, stats.get(key));
        }
    }

    /**
     * <summary>Populates parent element with stats data. The display name of metrics are used as children element
     * names.</summary>
     *
     * @param doc the owner document of parent element
     * @param parent Element to populate with stats data
     * @param stats the StatsResult with data to populate into element
     */
    private void storeStatsAsXml(Document doc, Element parent, StatsResult stats) {
        List metrics = stats.getMetrics();
        // for each item in stats result, create an element
        for (Iterator iter = metrics.iterator(); iter.hasNext();) {
            Metric metric = (Metric) iter.next();
            Element elem = doc.createElement(metric.getDisplayName());
            elem.appendChild(doc.createTextNode(String.valueOf(stats.getStat(metric))));
            parent.appendChild(elem);
        }
    }

}
