/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.matcher;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.SignatureChecker;
import com.topcoder.file.type.TextSignatureChecker;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for MimeMatcher class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcherTests extends TestCase {

    /** A mime string used in tests. */
    private static final String PDF_MIME = "application/pdf";

    /** Another mime string used in tests. */
    private static final String ZIP_MIME = "application/zip";

    /** The matcher instance that is tested. */
    private static final MimeMatcher MATCHER = new MimeMatcher(PDF_MIME);

    /**
     * Creation test for MimeMatcher(String) constructor with valid argument.
     */
    public void testMimeMatcherString1() {
        MimeMatcher matcher = new MimeMatcher(PDF_MIME);

        assertEquals("the extension were not propagated correctly", getMimes(matcher), new HashSet(Arrays
                .asList(new String[] {PDF_MIME})));
    }

    /**
     * Returns "mimes" field value from an MimeMatcher by reflection.
     *
     * @param matcher MimeMatcher to get the field value of
     * @return the value of "mimes" field of the given matcher
     */
    private static Set getMimes(MimeMatcher matcher) {
        return (Set) TestHelper.getFieldValue(matcher, MimeMatcher.class, "mimes");
    }

    /**
     * Creation test for MimeMatcher(String) with null argument. NullPointerException should be thrown.
     */
    public void testMimeMatcherString2() {
        try {
            new MimeMatcher((String) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for MimeMatcher(String) constructor with empty string argument. IllegalArgumentException
     * should be thrown.
     */
    public void testMimeMatcherString3() {
        try {
            new MimeMatcher(" ");
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test for MimeMatcher(List) constructor with valid argument.
     */
    public void testMimeMatcherList1() {
        List extList = Arrays.asList(new String[] {PDF_MIME, ZIP_MIME});

        MimeMatcher matcher = new MimeMatcher(extList);

        assertEquals("the extensions were not propagated correctly", getMimes(matcher), new HashSet(extList));
    }

    /**
     * Creation test for MimeMatcher(List) with null argument. NullPointerException should be thrown.
     */
    public void testMimeMatcherList2() {
        try {
            new MimeMatcher((List) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for MimeMatcher(List) with invalid list. IllegalArgumentException should be thrown.
     *
     * The following cases are verified: empty list, list contains null, list contains non-String instance, list
     * contains empty string.
     */
    public void testMimeMatcherList3() {
        List emptyList = Arrays.asList(new String[] {});
        List containsNullList = Arrays.asList(new String[] {PDF_MIME, null});
        List containsNonStringList = Arrays.asList(new Object[] {new Object(), ZIP_MIME});
        List containsEmptyStringList = Arrays.asList(new Object[] {PDF_MIME, ""});
        List[] lists = new List[] {emptyList, containsNullList, containsNonStringList, containsEmptyStringList};

        // test for each list
        for (int i = 0; i < lists.length; i++) {
            try {
                new MimeMatcher(lists[i]);
                fail("IllegalArgumentException should be thrown for list: " + lists[i]);
            } catch (IllegalArgumentException e) {
                // ok
            }
        }
    }

    /**
     * Creation test for MimeMatcher(Property) constructor with valid argument.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testMimeMatcherProperty1() throws ConfigurationException {
        // test single-valued "types" subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(MimeMatcher.MIMES_PROPERTY_KEY, PDF_MIME));
        MimeMatcher matcher = new MimeMatcher(property);

        assertEquals("the extension were not propagated correctly", getMimes(matcher), new HashSet(Arrays
                .asList(new String[] {PDF_MIME})));

        // test multi-valued "types" subproperty
        property = new Property("name");
        TestHelper.addSubproperty(property, new Property(MimeMatcher.MIMES_PROPERTY_KEY, new String[] {PDF_MIME,
            ZIP_MIME}));
        matcher = new MimeMatcher(property);

        assertEquals("the extensions were not propagated correctly", getMimes(matcher), new HashSet(Arrays
                .asList(new String[] {PDF_MIME, ZIP_MIME})));
    }

    /**
     * Creation test for MimeMatcher(Property) with null argument. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testMimeMatcherProperty2() throws ConfigurationException {
        try {
            new MimeMatcher((Property) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for MimeMatcher(Property) with invalid property. ConfigurationException should be thrown.
     *
     * The following cases are verified: property without types subproperty, empty types subproperty, types
     * subproperty contains empty string
     */
    public void testMimeMatcherProperty3() {
        // property without "types" subproperty
        try {
            new MimeMatcher(new Property("name"));
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

        // empty "types" subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(MimeMatcher.MIMES_PROPERTY_KEY));
        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

        // "types" subproperty contains empty string
        property = new Property("name");
        TestHelper.addSubproperty(property, new Property(MimeMatcher.MIMES_PROPERTY_KEY, ""));
        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }
    }

    /**
     * Tests match(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws FileMatchingException to JUnit
     * @throws IOException to JUnit
     */
    public void testMatchFile1() throws FileMatchingException, IOException {
        try {
            MATCHER.match((File) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match(FileType) method with null argument. NullPointerException should be thrown.
     */
    public void testMatchFileType1() {
        try {
            MATCHER.match((FileType) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match(FileType) method with valid arguments. Only the file type with name contained in MimeMatcher's
     * types list should be matched.
     */
    public void testMatchFileType2() {
        MimeMatcher matcher = new MimeMatcher(Arrays.asList(new String[] {PDF_MIME}));

        assertTrue("file type name should be matched", matcher.match(createFileMime(PDF_MIME)));
        assertFalse("file type name should not be matched", matcher.match(createFileMime(ZIP_MIME)));

        matcher = new MimeMatcher(Arrays.asList(new String[] {PDF_MIME, ZIP_MIME}));

        assertTrue("file type name should be matched", matcher.match(createFileMime(PDF_MIME)));
        assertTrue("file type name should not be matched", matcher.match(createFileMime(ZIP_MIME)));
    }

    /**
     * Tests match(File) method with valid arguments. Only the files which type name is contained in MimeMatcher's
     * types list should be matched.
     *
     * @throws IOException to JUnit
     * @throws FileMatchingException to JUnit
     */
    public void testMatchFile2() throws FileMatchingException, IOException {
        MimeMatcher matcher = new MimeMatcher(Arrays.asList(new String[] {PDF_MIME}));

        assertTrue("pdf file should be matched", matcher.match(new File("test_files/a/reqspec.pdf")));
        assertFalse("zip file should not be matched", matcher.match(new File("test_files/a/p.zip")));
        assertFalse("unrecognized type file should not be matched", matcher.match(new File("test_files/a/p.txt")));

        matcher = new MimeMatcher(Arrays.asList(new String[] {PDF_MIME, ZIP_MIME}));

        assertTrue("pdf file should be matched", matcher.match(new File("test_files/a/reqspec.pdf")));
        assertTrue("zip file should not be matched", matcher.match(new File("test_files/a/p.zip")));
        assertFalse("unrecognized type file should not be matched", matcher.match(new File("test_files/a/p.txt")));
    }

    /**
     * Tests match(File) method with invalid file argument. IOException should be thrown.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatchFile3() throws FileMatchingException {
        // directory
        try {
            MATCHER.match(new File("test_files/p/"));
            fail("IOException should be thrown.");
        } catch (IOException e) {
            // ok
        }
        // not existing
        try {
            MATCHER.match(new File("test_files/not_exists"));
            fail("IOException should be thrown.");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Returns a FileType instance with given mime. The other FileType arguments is unimportant.
     *
     * @param mime String with mime of file type
     * @return a FileType instance with given mime
     */
    private static FileType createFileMime(String mime) {
        return new FileType("name", new String[] {}, new String[] {}, mime,
                new SignatureChecker[] {new TextSignatureChecker()});
    }

}
