/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.matcher;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.SignatureChecker;
import com.topcoder.file.type.TextSignatureChecker;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for TypeNameMatcher class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TypeNameMatcherTests extends TestCase {

    /** A type name string used in tests. */
    private static final String PDF_TYPE_NAME = "Portable Document Format document";

    /** Another type name string used in tests. */
    private static final String ZIP_TYPE = "ZIP archive";

    /** The matcher instance that is tested. */
    private static final TypeNameMatcher MATCHER = new TypeNameMatcher(PDF_TYPE_NAME);

    /**
     * Creation test for TypeNameMatcher(String) constructor with valid argument.
     */
    public void testTypeNameMatcherString1() {
        TypeNameMatcher matcher = new TypeNameMatcher(PDF_TYPE_NAME);

        assertEquals("the extension were not propagated correctly", getTypes(matcher), new HashSet(Arrays
                .asList(new String[] {PDF_TYPE_NAME})));
    }

    /**
     * Returns "types" field value from an TypeNameMatcher by reflection.
     *
     * @param matcher TypeNameMatcher to get the field value of
     * @return the value of "types" field of the given matcher
     */
    private static Set getTypes(TypeNameMatcher matcher) {
        return (Set) TestHelper.getFieldValue(matcher, TypeNameMatcher.class, "types");
    }

    /**
     * Creation test for TypeNameMatcher(String) with null argument. NullPointerException should be thrown.
     */
    public void testTypeNameMatcherString2() {
        try {
            new TypeNameMatcher((String) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for TypeNameMatcher(String) constructor with empty string argument. IllegalArgumentException
     * should be thrown.
     */
    public void testTypeNameMatcherString3() {
        try {
            new TypeNameMatcher(" ");
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test for TypeNameMatcher(List) constructor with valid argument.
     */
    public void testTypeNameMatcherList1() {
        List extList = Arrays.asList(new String[] {PDF_TYPE_NAME, ZIP_TYPE});

        TypeNameMatcher matcher = new TypeNameMatcher(extList);

        assertEquals("the extensions were not propagated correctly", getTypes(matcher), new HashSet(extList));
    }

    /**
     * Creation test for TypeNameMatcher(List) with null argument. NullPointerException should be thrown.
     */
    public void testTypeNameMatcherList2() {
        try {
            new TypeNameMatcher((List) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for TypeNameMatcher(List) with invalid list. IllegalArgumentException should be thrown.
     *
     * The following cases are verified: empty list, list contains null, list contains non-String instance, list
     * contains empty string.
     */
    public void testTypeNameMatcherList3() {
        List emptyList = Arrays.asList(new String[] {});
        List containsNullList = Arrays.asList(new String[] {PDF_TYPE_NAME, null});
        List containsNonStringList = Arrays.asList(new Object[] {new Object(), ZIP_TYPE});
        List containsEmptyStringList = Arrays.asList(new Object[] {PDF_TYPE_NAME, ""});
        List[] lists = new List[] {emptyList, containsNullList, containsNonStringList, containsEmptyStringList};

        // test for each list
        for (int i = 0; i < lists.length; i++) {
            try {
                new TypeNameMatcher(lists[i]);
                fail("IllegalArgumentException should be thrown for list: " + lists[i]);
            } catch (IllegalArgumentException e) {
                // ok
            }
        }
    }

    /**
     * Creation test for TypeNameMatcher(Property) constructor with valid argument.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testTypeNameMatcherProperty1() throws ConfigurationException {
        // test single-valued "types" subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(TypeNameMatcher.TYPES_PROPERTY_KEY, PDF_TYPE_NAME));
        TypeNameMatcher matcher = new TypeNameMatcher(property);

        assertEquals("the extension were not propagated correctly", getTypes(matcher), new HashSet(Arrays
                .asList(new String[] {PDF_TYPE_NAME})));

        // test multi-valued "types" subproperty
        property = new Property("name");
        TestHelper.addSubproperty(property, new Property(TypeNameMatcher.TYPES_PROPERTY_KEY, new String[] {
            PDF_TYPE_NAME, ZIP_TYPE}));
        matcher = new TypeNameMatcher(property);

        assertEquals("the extensions were not propagated correctly", getTypes(matcher), new HashSet(Arrays
                .asList(new String[] {PDF_TYPE_NAME, ZIP_TYPE})));
    }

    /**
     * Creation test for TypeNameMatcher(Property) with null argument. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testTypeNameMatcherProperty2() throws ConfigurationException {
        try {
            new TypeNameMatcher((Property) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for TypeNameMatcher(Property) with invalid property. ConfigurationException should be thrown.
     *
     * The following cases are verified: property without types subproperty, empty types subproperty, types
     * subproperty contains empty string
     */
    public void testTypeNameMatcherProperty3() {
        // property without "types" subproperty
        try {
            new TypeNameMatcher(new Property("name"));
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

        // empty "types" subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(TypeNameMatcher.TYPES_PROPERTY_KEY));
        try {
            new TypeNameMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

        // "types" subproperty contains empty string
        property = new Property("name");
        TestHelper.addSubproperty(property, new Property(TypeNameMatcher.TYPES_PROPERTY_KEY, ""));
        try {
            new TypeNameMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }
    }

    /**
     * Tests match(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws FileMatchingException to JUnit
     * @throws IOException to JUnit
     */
    public void testMatchFile1() throws FileMatchingException, IOException {
        try {
            MATCHER.match((File) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match(FileType) method with null argument. NullPointerException should be thrown.
     */
    public void testMatchFileType1() {
        try {
            MATCHER.match((FileType) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match(FileType) method with valid arguments. Only the file type with name contained in
     * TypeNameMatcher's types list should be matched.
     */
    public void testMatchFileType2() {
        TypeNameMatcher matcher = new TypeNameMatcher(Arrays.asList(new String[] {PDF_TYPE_NAME}));

        assertTrue("file type name should be matched", matcher.match(createFileType(PDF_TYPE_NAME)));
        assertFalse("file type name should not be matched", matcher.match(createFileType(ZIP_TYPE)));

        matcher = new TypeNameMatcher(Arrays.asList(new String[] {PDF_TYPE_NAME, ZIP_TYPE}));

        assertTrue("file type name should be matched", matcher.match(createFileType(PDF_TYPE_NAME)));
        assertTrue("file type name should not be matched", matcher.match(createFileType(ZIP_TYPE)));
    }

    /**
     * Tests match(File) method with valid arguments. Only the files which type name is contained in
     * TypeNameMatcher's types list should be matched.
     *
     * @throws IOException to JUnit
     * @throws FileMatchingException to JUnit
     */
    public void testMatchFile2() throws FileMatchingException, IOException {
        TypeNameMatcher matcher = new TypeNameMatcher(Arrays.asList(new String[] {PDF_TYPE_NAME}));

        assertTrue("pdf file should be matched", matcher.match(new File("test_files/a/reqspec.pdf")));
        assertFalse("zip file should not be matched", matcher.match(new File("test_files/a/p.zip")));
        assertFalse("unrecognized type file should not be matched", matcher.match(new File("test_files/a/p.txt")));

        matcher = new TypeNameMatcher(Arrays.asList(new String[] {PDF_TYPE_NAME, ZIP_TYPE}));

        assertTrue("pdf file should be matched", matcher.match(new File("test_files/a/reqspec.pdf")));
        assertTrue("zip file should not be matched", matcher.match(new File("test_files/a/p.zip")));
        assertFalse("unrecognized type file should not be matched", matcher.match(new File("test_files/a/p.txt")));
    }

    /**
     * Tests match(File) method with invalid file argument. IOException should be thrown.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatchFile3() throws FileMatchingException {
        // directory
        try {
            MATCHER.match(new File("test_files/p/"));
            fail("IOException should be thrown.");
        } catch (IOException e) {
            // ok
        }
        // not existing
        try {
            MATCHER.match(new File("test_files/not_exists"));
            fail("IOException should be thrown.");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Returns a FileType instance with given name. The other FileType arguments is unimportant.
     *
     * @param name String with name of file type
     * @return a FileType instance with given name
     */
    private static FileType createFileType(String name) {
        return new FileType(name, new String[] {}, new String[] {}, "",
                new SignatureChecker[] {new TextSignatureChecker()});
    }

}
