/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.reporting;

import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;

/**
 * <p>Unit test case for StatsCollection class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class StatsCollectionTests extends TestCase {

    /** A key used in tests. */
    private static final String KEY = "test key";

    /** Another key used in tests. */
    private static final String KEY_2 = "test key 2";

    /** A metric used in tests. */
    private static final Metric METRIC = new Metric("name", "test display name", "test description");

    /** Another metric used in tests. */
    private static final Metric METRIC_2 = new Metric("name 2", "test display name", "test description");

    /** An instance of stats result used in tests. */
    private StatsResult statsResult;

    /** Another instance of stats result used in tests. */
    private StatsResult statsResult2;

    /** The instance for tests. */
    private StatsCollection stats;

    /**
     * Sets up the test. Initializes tested instance.
     */
    protected void setUp() {
        stats = new StatsCollection();
        statsResult = new StatsResult();
        statsResult.setStat(METRIC, 100);
        stats.add(KEY, statsResult);

        statsResult2 = new StatsResult();
        statsResult2.setStat(METRIC, 10);
        statsResult2.setStat(METRIC_2, 20);
    }

    /**
     * Creation test for StatsCollection default constructor.
     */
    public void testStatsCollection1() {
        StatsCollection newStats = new StatsCollection();
        assertTrue("new stats result should be empty", newStats.getAllKeys().isEmpty());
    }

    /**
     * Tests add method with some null argument. NullPointerException should be thrown.
     */
    public void testAdd1() {
        try {
            stats.add(null, statsResult2);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }

        try {
            stats.add(KEY, null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests add method with empty string key. IllegalArgumentException should be thrown.
     */
    public void testAdd2() {
        try {
            stats.add(" ", statsResult2);
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests add method with existing string key. Stats result should be aggregated with current stats.
     */
    public void testAdd3() {
        stats.add(KEY, statsResult2);
        assertEquals("stats result was not added correctly", stats.get(KEY).getStat(METRIC), 110);
        assertEquals("stats result was not added correctly", stats.get(KEY).getStat(METRIC_2), 20);
    }

    /**
     * Tests add method with non-existing string key. A copy of stats result should be inserted under that key.
     */
    public void testAdd4() {
        stats.add(KEY_2, statsResult2);
        assertEquals("stats result was not added correctly", stats.get(KEY_2).getStat(METRIC), 10);
        assertEquals("stats result was not added correctly", stats.get(KEY_2).getStat(METRIC_2), 20);
        assertNotSame("stats result was not cloned", stats.get(KEY_2), statsResult2);
        assertEquals("stats result for other key should remain unchanged", stats.get(KEY).getStat(METRIC), 100);
    }

    /**
     * Tests get(key) method with null argument. NullPointerException should be thrown.
     */
    public void testGet1() {
        try {
            stats.get(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests get(key) method with empty string argument. IllegalArgumentException should be thrown.
     */
    public void testGet2() {
        try {
            stats.get(" ");
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests get(key) method with existing key. Stats result should be returned.
     */
    public void testGet3() {
        assertEquals("stats result was not retrieved correctly", stats.get(KEY).getStat(METRIC), 100);
    }

    /**
     * Tests get(key) method with non-existing key. Null should be returned.
     */
    public void testGet4() {
        assertNull("there should be no stats result", stats.get(KEY_2));
    }

    /**
     * Tests getAllKeys() method.
     */
    public void testGetAllKeys1() {
        List keys = stats.getAllKeys();
        assertEquals("all keys list was retrieved incorrectly", keys, Arrays.asList(new String[] {KEY}));

        // verify that changes in stats are not reflected in the returned list
        stats.add(KEY_2, statsResult2);
        assertEquals("all keys list should remain unaffected", keys, Arrays.asList(new String[] {KEY}));
    }

    /**
     * Tests getAllKeys() method. Verifies that returned list is in alphabetical order (case insensitive).
     */
    public void testGetAllKeys2() {
        stats.clear();
        stats.add("Bca", statsResult);
        stats.add("cab", statsResult);
        stats.add("a", statsResult);
        stats.add("cba", statsResult);
        stats.add("bac", statsResult);
        stats.add("abc", statsResult);
        assertEquals("all keys list should be sorted", stats.getAllKeys(), Arrays.asList(new String[] {"a", "abc",
            "bac", "Bca", "cab", "cba"}));
    }

    /**
     * Tests clear() method.
     */
    public void testClear() {
        stats.clear();
        assertTrue("stats should be empty now", stats.getAllKeys().isEmpty());
    }

}
