/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

import com.topcoder.util.datavalidator.ObjectValidator;

/**
 * <summary> ArgumentOption is a pure data oriented class which packs together the siwtch name and help message,
 * required flag and an optional validator for an option. An ArgumentHandler can propose multiple options to the
 * command line interface. </summary>
 *
 * <thread-safety> This class is immutable and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 2.0b
 */
public class ArgumentOption {

    /**
     * <summary> The switch name for this option. Swith name unique identifies an argument. But switch name does
     * not need to be unique among all proposals because the arguments could be shared (for instance we are very
     * likely to configure a FileMatcher twice). </summary>
     *
     */
    private final String switchName;

    /**
     * <summary> This is the helper message to be printed to the user. It's free style text but should normally
     * respect the format of the help message screen of the respective implementation. </summary>
     *
     */
    private final String helpMessage;

    /**
     * <summary> A flag indicates whether the switch is required or not. Notice it is only concerned with the
     * switch rather than any actual value after the switch. </summary>
     *
     */
    private final boolean required;

    /**
     * <summary> An optional validator to validate the argument value at the command line interface level. This
     * validation is not required and ArgumentHandler can perform more validation when the arguments are received.
     * </summary>
     *
     */
    private final ObjectValidator validator;

    /**
     * <summary> Constructor with 3 of the arguments (with validator set to null). </summary>
     *
     * @param switchName the switch name
     * @param helpMessage the help message
     * @param required the required flag
     *
     * @exception NullPointerException if switchName or helpMessage is null
     * @exception IllegalArgumentException if switchName or helpMessage is empty after trimmed
     */
    public ArgumentOption(String switchName, String helpMessage, boolean required) {
        CheckHelper.checkString(switchName, "switchName");
        CheckHelper.checkString(helpMessage, "helpMessage");
        this.switchName = switchName;
        this.helpMessage = helpMessage;
        this.required = required;
        this.validator = null;
    }

    /**
     * <summary> Constructor with all arguments. </summary>
     *
     * @param switchName the switch name
     * @param helpMessage the help message
     * @param required the required flag
     * @param validator the object validator
     *
     * @exception NullPointerException if switchName or helpMessage or validator is null
     * @exception IllegalArgumentException if switchName or helpMessage is empty after trimmed
     */
    public ArgumentOption(String switchName, String helpMessage, boolean required,
        ObjectValidator validator) {
        CheckHelper.checkString(switchName, "switchName");
        CheckHelper.checkString(helpMessage, "helpMessage");
        CheckHelper.checkNull(validator, "validator");
        this.switchName = switchName;
        this.helpMessage = helpMessage;
        this.required = required;
        this.validator = validator;
    }

    /**
     * <summary> Get the switch name. </summary>
     *
     * @return the switch name
     */
    public String getSwitchName() {
        return switchName;
    }

    /**
     * <summary> Get the help message. </summary>
     *
     * @return the help message
     */
    public String getHelpMessage() {
        return helpMessage;
    }

    /**
     * <summary> Get the required flag. </summary>
     *
     * @return the required flag
     */
    public boolean isRequired() {
        return required;
    }

    /**
     * <summary> Get the object validator. May be null.</summary>
     *
     * @return the object validator
     */
    public ObjectValidator getValidator() {
        return validator;
    }

}
