/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.topcoder.util.commandline.ArgumentValidationException;
import com.topcoder.util.commandline.ArgumentValidator;
import com.topcoder.util.commandline.CommandLineUtility;
import com.topcoder.util.commandline.Switch;
import com.topcoder.util.commandline.UsageException;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.datavalidator.ObjectValidator;

/**
 * <summary> The command line entry point of the File Statistics utility. It uses ExtendedFileStatistics to fulfill
 * the task. When argument is invalid help message should be printed. </summary>
 *
 * <arguments> java Main [-c config-file] [-n namespace] [-b base-path] [-r] [-o output-file] path [path2 path3
 * ...]
 *
 * <br/> -c config-file: provides a config file to initialize from. It should be in config XML format and is loaded
 * to the namespace (provided or default).
 *
 * <br/> -n namespace: provides a namespace to initialize from If neither is provided the default namespace is
 * used. Provided namespace should be preloaded.
 *
 * <br/> -b base-path: all path will be relative to it.
 *
 * <br/> -r: whether to recursively traverse sub-directory.
 *
 * <br/> -o output-file: the output filename. If missing output is printed to console. </arguments>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * <changes> Since version 2.0b plugins can register command line options. This is implemented by the
 * ArgumentHandlers. This class handles the logic to collect the proposals, to validate and to send the arguments
 * to the handlers. </changes>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 */
public class Main {

    /**
     * <summary> The namespace to used if no namespace is provided. </summary>
     */
    public static final String NAMESPACE = "com.topcoder.file.statistics.Main";

    /**
     * <summary> Private constructor to disallow instantiation. </summary>
     */
    private Main() {
        // empty
    }

    /**
     * <summary> Command line entry. Refer to class level doc for options. The options are parsed by Command Line
     * Utility component. </summary>
     *
     * <changes> This method is changed to handle the configurable arguments. See configureArguments() private
     * method for more info. </changes>
     *
     * @param args the command line args
     */
    public static void main(String[] args) {

        // without arguments, only print the usage help
        if (args.length <= 1) {
            printHelp(Collections.EMPTY_LIST);
            return;
        }

        List argumentOptions = new ArrayList();

        try {
            // configure command line options
            CommandLineUtility utility = new CommandLineUtility();
            utility.addSwitch(new Switch("c", false, 1, 1, null));
            utility.addSwitch(new Switch("n", false, 1, 1, null));
            utility.addSwitch(new Switch("b", false, 1, 1, null));
            utility.addSwitch(new Switch("r", false, 0, 0, null));
            utility.addSwitch(new Switch("o", false, 1, 1, null));
            // parse arguments into option values
            try {
                utility.parse(args);
            } catch (UsageException e) {
                // since 2.0b; ok, because an additional switch can be supplied
            }
            String configFile = utility.getSwitch("c").getValue();
            String namespace = utility.getSwitch("n").getValue();
            String basePath = utility.getSwitch("b").getValue();
            String outputFile = utility.getSwitch("o").getValue();
            boolean recursive = utility.getSwitch("r").getValues() != null;
            String[] paths = (String[]) utility.getParameters().toArray(new String[0]);

            // if namespace was not provided, use default
            if (namespace == null) {
                namespace = NAMESPACE;
            }

            // if present, load config file to the namespace
            if (configFile != null) {
                ConfigManager.getInstance().add(namespace, configFile, ConfigManager.CONFIG_XML_FORMAT);
            }

            try {

                // create file statistics and configure the base path
                ExtendedFileStatistics statistics = new ExtendedFileStatistics(namespace);
                if (basePath != null) {
                    statistics.getReporting().reset(new File(basePath));
                }

                // Get the list of ArgumentHandlers from the FileStatistics instance created, and
                // perform the version 2.0b enhancement logic
                configureArguments(utility, statistics.getArgumentHandlers(), args, argumentOptions);

                // proceeds with 1.0 version logic

                // process all file and directory paths
                for (int i = 0; i < paths.length; i++) {
                    String path = paths[i];
                    File file = new File(basePath, path);
                    if (file.isDirectory()) {
                        statistics.processDirectory(file, recursive);
                    } else {
                        statistics.processFile(file);
                    }
                }

                // write report to file or console
                if (outputFile != null) {
                    statistics.printReport(new File(outputFile));
                } else {
                    statistics.printReport();
                }

            } finally {
                // remove loaded namespace
                if (configFile != null) {
                    ConfigManager cm = ConfigManager.getInstance();
                    if (cm.existsNamespace(namespace)) {
                        cm.removeNamespace(namespace);
                    }
                }
            }

        } catch (Exception e) {
            e.printStackTrace();

            // use a list of known ArgumentOption instances to print help message.
            // The print does not include any duplicate ArgumentOptions (by switch name)
            printHelp(removeDuplicateSwitchNames(argumentOptions));
        }

    }

    /**
     * <summary> Validates, extracts values and configures handlers for the configurable arguments.
     *
     * a) Collect the proposals from the ArgumentHandler instances.
     *
     * b) Validate the argument value with each of them. If the switch is required but does not exist, error is
     * shown. Also if the switch exists but the follow-up value ("" if no follow-up value) does not pass the
     * validator.
     *
     * c) When necessary uses the list of ArgumentOption instances to print help message. The print will not
     * include duplicate ArgumentOptions (switch name).
     *
     * d) When all validation passed, collect necessary arguments for each handler and configure the handle. Notice
     * this process can still result in validation error. </summary>
     *
     *
     * @param utility the CommandLineUtility in use
     * @param argumentHandlers List of ArgumentHandler-s
     * @param args command line arguments
     * @param argumentOptions List of argumentOptions to be fiilled up
     *
     * @throws Exception if any exception occurs
     *
     * @since 2.0b
     */
    private static void configureArguments(CommandLineUtility utility, List argumentHandlers, String[] args,
        List argumentOptions) throws Exception {
        // Collect the proposals from the ArgumentHandler instances.
        for (Iterator iter = argumentHandlers.iterator(); iter.hasNext();) {
            ArgumentHandler argumentHandler = (ArgumentHandler) iter.next();
            argumentOptions.addAll(argumentHandler.proposeOptions());
        }

        // if there is not argument handlers or argument options, nothing is to be done
        if (argumentOptions.isEmpty()) {
            return;
        }

        // validate and parse the additional arguments
        for (Iterator iter = argumentOptions.iterator(); iter.hasNext();) {
            ArgumentOption argument = (ArgumentOption) iter.next();

            // create argument validator
            ObjectValidator dataValidator = argument.getValidator();
            ArgumentValidator argValidator = dataValidator == null ? null : createValidatorAdapter(dataValidator);
            utility.addSwitch(new Switch(argument.getSwitchName(), argument.isRequired(), -1, -1, argValidator));
        }
        utility.parse(args);

        // when the validation is passed, collect necessary arguments for each handler and configure them.
        for (Iterator iter = argumentHandlers.iterator(); iter.hasNext();) {
            ArgumentHandler argumentHandler = (ArgumentHandler) iter.next();

            // extract argument values map for each proposed options for this handler
            Map argMap = new HashMap();
            for (Iterator argOptIter = argumentHandler.proposeOptions().iterator(); argOptIter.hasNext();) {
                ArgumentOption argument = (ArgumentOption) argOptIter.next();
                Switch argSwitch = utility.getSwitch(argument.getSwitchName());
                if (argSwitch.getValue() != null) {
                    // the switch has a value
                    argMap.put(argument.getSwitchName(), argSwitch.getValue());
                } else if (argSwitch.getValues() != null) {
                    // the switch is included in command line without any follow-up value
                    argMap.put(argument.getSwitchName(), "");
                }
            }
            // configure handler; notice this may also throw an exception
            argumentHandler.handleArguments(argMap);
        }
    }

    /**
     * <summary>Creates and returns an ArgumentValidator adapter for given ObjectValidator instance. </summary>
     *
     * @param dataValidator ObjectValidator to adapt, not null
     * @return adapted ArgumentValidator
     */
    private static ArgumentValidator createValidatorAdapter(final ObjectValidator dataValidator) {
        return new ArgumentValidator() {
            public void validate(String arg) throws ArgumentValidationException {
                if (!dataValidator.valid(arg)) {
                    throw new ArgumentValidationException(arg, dataValidator.getMessage(arg));
                }
            }
        };
    }

    /**
     * <summary> Removes duplicate ArgumentOptions (by switch name) from the list.</summary>
     *
     * @param argumentOptions list of ArgumentOption instances.
     * @return list of ArgumentOption instances without duplicates
     */
    private static List removeDuplicateSwitchNames(List argumentOptions) {
        List result = new ArrayList();
        Set names = new HashSet();
        // for each argument option on the list, if there was no such switch name earlier, add to result list
        for (Iterator iter = names.iterator(); iter.hasNext();) {
            ArgumentOption argumentOption = (ArgumentOption) iter.next();
            if (!names.contains(argumentOption.getSwitchName())) {
                names.add(argumentOption.getSwitchName());
                result.add(argumentOption);
            }
        }
        return result;
    }

    /**
     * <summary> Print help messages to console. A detailed usage info is written to standard output. </summary>
     *
     * <changes> Since version 2.0b this method takes a list of ArgumentOption instances and prints the switch name
     * and help message to the user in addition to the original information. </changes>
     *
     * @param argumentOptions List of ArgumentOption instances to be printed in addition to standard options
     */
    private static void printHelp(List argumentOptions) {
        PrintStream out = System.out;
        // print standard usage information
        out.println("java " + Main.class.getName());
        out.println("\t  [-c config-file] [-n namespace] [-b base-path]"
            + " [-r] [-o output-file] path [path2 path3 ...]");
        out.println("-c config-file: provides a config file to initialize from");
        out.println("It should be in config XML format and is loaded to the namespace (provided or default)");
        out.println("-n namespace: provides a namespace to initialize from");
        out.println("If neither is provided the default namespace is used. "
            + "Provided namespace should be preloaded.");
        out.println("-b base-path: all path will be relative to it.");
        out.println("-r: whether to recursively traverse sub-directory.");
        out.println("-o output-file: the output filename. If missing output is printed to console.");

        // print argument options
        if (!argumentOptions.isEmpty()) {
            out.println("Additional argument options:");
            // foreach argument option, print switch name and help message
            for (Iterator iter = argumentOptions.iterator(); iter.hasNext();) {
                ArgumentOption argOpt = (ArgumentOption) iter.next();
                out.println(argOpt.getSwitchName() + " : " + argOpt.getHelpMessage() + " "
                    + (argOpt.isRequired() ? "(required)" : "(optional)"));
            }
        }
    }
}
