/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;

/**
 * <p>Unit test case for FileStatistics class.</p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 */
public class FileStatisticsTests extends TestCase {

    /** A file processor used for tests. */
    private static final FileProcessor FILE_PROCESSOR_1 = new CStyleLineCounter("test alias 1",
            new TextFileMatcher());

    /** Another file processor used for tests. */
    private static final FileProcessor FILE_PROCESSOR_2 = new TextLineCounter("test alias 2", new AnyFileMatcher());

    /** A stats reporting used for tests. */
    private static final StatsReporting REPORTING = new BasicStatsReporting();

    /** The statistics instance that is tested. */
    private FileStatistics statistics;

    /**
     * Sets up the test. Initializes tested FileStatistics instance.
     */
    protected void setUp() {
        statistics = new FileStatistics();
        statistics.addFileProcessor(FILE_PROCESSOR_1);
        statistics.setMatchAll(true);
        statistics.setReporting(REPORTING);
    }

    /**
     * Creation test for FileStatistics with default constructor.
     */
    public void testFileStatistics1() {
        FileStatistics fileStatistics = new FileStatistics();
        assertTrue("default file statistics should have no file processors", fileStatistics.getAllFileProcessors()
                .isEmpty());
        assertFalse("default file statistics should have matchAll flag disabled", fileStatistics.isMatchAll());
        StatsReporting reporting = fileStatistics.getReporting();
        assertTrue("default file statistcs should have XmlStatsReporting reporting", reporting != null
                && (reporting instanceof XmlStatsReporting));
    }

    /**
     * Creation test for FileStatistics with null namespace argument. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testFileStatistics2() throws ConfigurationException {
        try {
            new FileStatistics(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for FileStatistics with empty string namespace argument. IllegalArgumentException should be
     * thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testFileStatistics3() throws ConfigurationException {
        try {
            new FileStatistics(" ");
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test for FileStatistics with non-existing namespace argument. ConfigurationException should be
     * thrown.
     */
    public void testFileStatistics4() {
        try {
            new FileStatistics("non_existing");
            fail("ConfigurationException should be thrown.");
        } catch (ConfigurationException e) {
            // ok
        }
    }

    /**
     * Creation test for FileStatistics with valid configuration namespace argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testFileStatistics5a() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
                .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        try {
            FileStatistics newStatistics = new FileStatistics(Main.NAMESPACE);
            assertEquals("there should be two file processors loaded",
                    newStatistics.getAllFileProcessors().size(), 2);
            assertTrue("matchAll flag should be on", newStatistics.isMatchAll());
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for FileStatistics with valid configuration namespace argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testFileStatistics5b() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance().add(Main.NAMESPACE, "conf/FileStatistics_minimal1.xml",
                ConfigManager.CONFIG_XML_FORMAT);
        try {
            FileStatistics newStatistics = new FileStatistics(Main.NAMESPACE);
            assertEquals("there should be zero file processors loaded", newStatistics.getAllFileProcessors()
                    .size(), 0);
            assertFalse("matchAll flag should be off", newStatistics.isMatchAll());
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for FileStatistics with valid configuration namespace argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testFileStatistics5c() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance().add(Main.NAMESPACE, "conf/FileStatistics_minimal2.xml",
                ConfigManager.CONFIG_XML_FORMAT);
        try {
            FileStatistics newStatistics = new FileStatistics(Main.NAMESPACE);
            assertEquals("there should be one file processor loaded", newStatistics.getAllFileProcessors().size(),
                    1);
            assertFalse("matchAll flag should be off", newStatistics.isMatchAll());
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for FileStatistics with invalid configuration namespace arguments. ConfigurationException
     * should be thrown.
     *
     * @throws ConfigManagerException to JUnit
     */
    public void testFileStatistics6() throws ConfigManagerException {
        String[] cases = new String[] {"bad_matcher_classname", "bad_processor_classname",
            "bad_reporting_classname", "bad_reporting_stylesheet", "empty_alias", "empty_matcher_extensions",
            "miss_alias", "miss_matcher_classname", "miss_processor_classname", "miss_processors",
            "miss_reporting_classname"};

        for (int i = 0; i < cases.length; i++) {
            String file = "conf/FileStatistics_" + cases[i] + ".xml";
            ConfigManager.getInstance().add(Main.NAMESPACE, file, ConfigManager.CONFIG_XML_FORMAT);
            try {
                new FileStatistics(Main.NAMESPACE);
                fail("ConfigurationException should be thrown. case: " + cases[i]);
            } catch (ConfigurationException e) {
                // ok
            } finally {
                ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
            }
        }
    }

    /**
     * Tests addFileProcessor method with null argument. NullPointerException should be thrown.
     */
    public void testAddFileProcessor1() {
        try {
            statistics.addFileProcessor(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests addFileProcessor method with valid argument. Passed FileProcessor should be stored under its alias.
     * Adding existing file processor should cause it to be moved to the end of the processors list.
     */
    public void testAddFileProcessor2() {
        statistics.addFileProcessor(FILE_PROCESSOR_2);

        assertSame("file processor was not added properly", statistics.getFileProcessor(FILE_PROCESSOR_2
                .getAlias()), FILE_PROCESSOR_2);
        assertEquals("both file processors should be in statistics now", statistics.getAllFileProcessors(), Arrays
                .asList(new Object[] {FILE_PROCESSOR_1, FILE_PROCESSOR_2}));

        // adding existing file processor should cause its reinsertion at the end
        statistics.addFileProcessor(FILE_PROCESSOR_1);
        assertEquals("the first file processor should be reinserted at the end of the list", statistics
                .getAllFileProcessors(), Arrays.asList(new Object[] {FILE_PROCESSOR_2, FILE_PROCESSOR_1}));
    }

    /**
     * Tests removeFileProcessor method with null argument. NullPointerException should be thrown.
     */
    public void testRemoveFileProcessor1() {
        try {
            statistics.removeFileProcessor(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests removeFileProcessor method with empty string argument. IllegalArgumentException should be thrown.
     */
    public void testRemoveFileProcessor2() {
        try {
            statistics.removeFileProcessor(" ");
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests removeFileProcessor method with non-existing alias. Null should be returned.
     */
    public void testRemoveFileProcessor3() {
        FileProcessor processor = statistics.removeFileProcessor("non-existing alias");
        assertNull("there is no file processor under that alias", processor);
    }

    /**
     * Tests removeFileProcessor method with existing alias. The processor with that alias should be removed and
     * returned.
     */
    public void testRemoveFileProcessor4() {
        FileProcessor removedProcessor = statistics.removeFileProcessor(FILE_PROCESSOR_1.getAlias());
        assertNull("file processor has not been removed properly", statistics.getFileProcessor(FILE_PROCESSOR_1
                .getAlias()));
        assertSame("incorrect file processor was removed", FILE_PROCESSOR_1, removedProcessor);
    }

    /**
     * Tests getFileProcessor method with null argument. NullPointerException should be thrown.
     */
    public void testGetFileProcessor1() {
        try {
            statistics.getFileProcessor(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests getFileProcessor method with empty string argument. IllegalArgumentException should be thrown.
     */
    public void testGetFileProcessor2() {
        try {
            statistics.getFileProcessor(" ");
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests getFileProcessor method with non-existing alias. Null should be returned.
     */
    public void testGetFileProcessor3() {
        FileProcessor fileProcessor = statistics.getFileProcessor("non-existing alias");
        assertNull("there should be no file processor for non-existing alias", fileProcessor);
    }

    /**
     * Tests getFileProcessor method with existing alias. The processor with that alias should be returned.
     */
    public void testGetFileProcessor4() {
        FileProcessor fileProcessor = statistics.getFileProcessor(FILE_PROCESSOR_1.getAlias());
        assertSame("incorrect file processor was returned", FILE_PROCESSOR_1, fileProcessor);
    }

    /**
     * Tests getAllFileProcessors mthod. Verifies that an independent copy of file processors collection is
     * returned.
     */
    public void testGetAllFileProcessors() {
        List fileProcessors = statistics.getAllFileProcessors();
        assertEquals("incorrect fileProcessors list retrieved", fileProcessors, Arrays
                .asList(new Object[] {FILE_PROCESSOR_1}));

        // change retrieved list, file statistics should remain unaffected
        fileProcessors.add(FILE_PROCESSOR_2);
        assertEquals("incorrect fileProcessors list retrieved", statistics.getAllFileProcessors(), Arrays
                .asList(new Object[] {FILE_PROCESSOR_1}));
    }

    /**
     * Tests clearAllFileProcessors method. This should empty the file processors list.
     */
    public void testClearAllFileProcessors() {
        statistics.clearAllFileProcessors();
        assertTrue("the file processors list should be empty", statistics.getAllFileProcessors().isEmpty());
    }

    /**
     * Tests getReporting() getter method.
     */
    public void testGetReporting() {
        assertSame("stats reporting was not retrieved properly", statistics.getReporting(), REPORTING);
    }

    /**
     * Tests setReporting() setter method with null argument. NullPointerException should be thrown.
     */
    public void testSetReporting1() {
        try {
            statistics.setReporting(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests setReporting() setter method with valid argument.
     */
    public void testSetReporting2() {
        XmlStatsReporting reporting = new XmlStatsReporting();
        statistics.setReporting(reporting);
        assertSame("stats reporting was not set properly", statistics.getReporting(), reporting);
    }

    /**
     * Tests isMatchAll() getter method.
     */
    public void testIsMatchAll() {
        assertEquals("matchAll flag was not retrieved properly", statistics.isMatchAll(), true);
    }

    /**
     * Tests setMatchAll() setter method.
     */
    public void testSetMatchAll() {
        statistics.setMatchAll(false);
        assertFalse("matchAll flag was not set properly", statistics.isMatchAll());
        statistics.setMatchAll(true);
        assertTrue("matchAll flag was not set properly", statistics.isMatchAll());
    }

    /**
     * Tests processFile() method with null argument. NullPointerException should be thrown.
     *
     * @throws IOException to JUnit
     * @throws FileStatisticsException to JUnit
     */
    public void testProcessFile1() throws FileStatisticsException, IOException {
        try {
            statistics.processFile(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests processFile() method with valid argument.
     *
     * @throws IOException to JUnit
     * @throws FileStatisticsException to JUnit
     */
    public void testProcessFile2() throws FileStatisticsException, IOException {
        statistics.processFile(new File("test_files/a/p.txt"));
        statistics.processFile(new File("test_files/b/q.jav"));
    }

    /**
     * Tests processFile() method with non-existing file. For this file statistics, TextFileMatcher will throw
     * FileNotFoundException.
     *
     * @throws IOException to JUnit
     * @throws FileStatisticsException to JUnit
     */
    public void testProcessFile3() throws FileStatisticsException, IOException {
        try {
            statistics.processFile(new File("test_files/a/dont_exist"));
            fail("FileNotFoundException should be thrown.");
        } catch (FileNotFoundException e) {
            // ok
        }
    }

    /**
     * Tests getArgumentHandlers() method. This is a package private method, but adding
     * one simple test won't be bad.
     *
     * @since 2.0b
     */
    public void testname() {
        List argHandlers = statistics.getArgumentHandlers();
        assertEquals("incorrect argumetn handlers list was obtained", argHandlers.size(), 1);
    }

}
