/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.matcher;

import java.io.File;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for ExtensionMatcher class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ExtensionMatcherTests extends TestCase {

    /** A file extension string used in tests. */
    private static final String EXTENSION_TXT = "txt";

    /** The matcher instance that is tested. */
    private static final ExtensionMatcher MATCHER = new ExtensionMatcher(EXTENSION_TXT);

    /**
     * Creation test for ExtensionMatcher(String) constructor with valid argument.
     */
    public void testExtensionMatcherString1() {
        ExtensionMatcher matcher = new ExtensionMatcher(EXTENSION_TXT);

        assertEquals("the extension were not propagated correctly", getExtensions(matcher), new HashSet(Arrays
                .asList(new String[] {EXTENSION_TXT})));
    }

    /**
     * Returns "extensions" field value from an ExtensionMatcher by reflection.
     *
     * @param matcher ExtensionMatcher to get the field value of
     * @return the value of "extensions" field of the given matcher
     */
    private static Set getExtensions(ExtensionMatcher matcher) {
        return (Set) TestHelper.getFieldValue(matcher, ExtensionMatcher.class, "extensions");
    }

    /**
     * Creation test for ExtensionMatcher(String) with null argument. NullPointerException should be thrown.
     */
    public void testExtensionMatcherString2() {
        try {
            new ExtensionMatcher((String) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for ExtensionMatcher(String) constructor with empty string argument. Empty string extension is
     * a valid one.
     */
    public void testExtensionMatcherString3() {
        ExtensionMatcher matcher = new ExtensionMatcher("");

        assertEquals("the extension were not propagated correctly", getExtensions(matcher), new HashSet(Arrays
                .asList(new String[] {""})));
    }

    /**
     * Creation test for ExtensionMatcher(List) constructor with valid argument.
     */
    public void testExtensionMatcherList1() {
        List extList = Arrays.asList(new String[] {EXTENSION_TXT, ""});

        ExtensionMatcher matcher = new ExtensionMatcher(extList);

        assertEquals("the extensions were not propagated correctly", getExtensions(matcher), new HashSet(extList));
    }

    /**
     * Creation test for ExtensionMatcher(List) with null argument. NullPointerException should be thrown.
     */
    public void testExtensionMatcherList2() {
        try {
            new ExtensionMatcher((List) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for ExtensionMatcher(List) with invalid list. IllegalArgumentException should be thrown.
     *
     * The following cases are verified: empty list, list contains null, list contains non-String instance.
     */
    public void testExtensionMatcherList3() {
        List emptyList = Arrays.asList(new String[] {});
        List containsNullList = Arrays.asList(new String[] {EXTENSION_TXT, null});
        List containsNonStringList = Arrays.asList(new Object[] {new Object(), ""});
        List[] lists = new List[] {emptyList, containsNullList, containsNonStringList};

        // test for each list
        for (int i = 0; i < lists.length; i++) {
            try {
                new ExtensionMatcher(lists[i]);
                fail("IllegalArgumentException should be thrown for list: " + lists[i]);
            } catch (IllegalArgumentException e) {
                // ok
            }
        }
    }

    /**
     * Creation test for ExtensionMatcher(Property) constructor with valid argument.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testExtensionMatcherProperty1() throws ConfigurationException {
        // test single-valued "extensions" subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(ExtensionMatcher.EXTENSIONS_PROPERTY_KEY, EXTENSION_TXT));
        ExtensionMatcher matcher = new ExtensionMatcher(property);

        assertEquals("the extension were not propagated correctly", getExtensions(matcher), new HashSet(Arrays
                .asList(new String[] {EXTENSION_TXT})));

        // test multi-valued "extensions" subproperty
        property = new Property("name");
        TestHelper.addSubproperty(property, new Property(ExtensionMatcher.EXTENSIONS_PROPERTY_KEY, new String[] {
            EXTENSION_TXT, ""}));
        matcher = new ExtensionMatcher(property);

        assertEquals("the extensions were not propagated correctly", getExtensions(matcher), new HashSet(Arrays
                .asList(new String[] {EXTENSION_TXT, ""})));
    }

    /**
     * Creation test for ExtensionMatcher(Property) with null argument. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testExtensionMatcherProperty2() throws ConfigurationException {
        try {
            new ExtensionMatcher((Property) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for ExtensionMatcher(Property) with invalid property. ConfigurationException should be thrown.
     *
     * The following cases are verified: property without extensions subproperty, empty extensions subproperty.
     */
    public void testExtensionMatcherProperty3() {
        // property without "extensions" subproperty
        try {
            new ExtensionMatcher(new Property("name"));
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

        // empty extensions subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(ExtensionMatcher.EXTENSIONS_PROPERTY_KEY));
        try {
            new ExtensionMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }
    }

    /**
     * Tests match(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatchFile1() throws FileMatchingException {
        try {
            MATCHER.match((File) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match(String) method with null argument. NullPointerException should be thrown.
     */
    public void testMatchString1() {
        try {
            MATCHER.match((String) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match methods with valid arguments. Filename strings with 'txt' extensions should be matched.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatch2() throws FileMatchingException {
        // positives
        String[] names = new String[] {"p.txt", "foo.bar.1.2-3.txt", ".txt", "test_files/a/p.txt",
            "test_files/non-existing.txt"};
        for (int i = 0; i < names.length; i++) {
            assertTrue("file name should be matched, name: " + names[i], MATCHER.match(names[i]));
            assertTrue("file name should be matched, name: " + names[i], MATCHER.match(new File(names[i])));
        }

        // negatives
        names = new String[] {"", "txt.", "a.tx", "a.txt.", "test_files/a", "test_files/non-existing"};
        for (int i = 0; i < names.length; i++) {
            assertFalse("file name should not be matched, name: " + names[i], MATCHER.match(names[i]));
            assertFalse("file name should not be matched, name: " + names[i], MATCHER.match(new File(names[i])));
        }
    }

    /**
     * Tests match methods with valid arguments with matcher accepting empty extension. Filename strings without or
     * ending with dot should be matched.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatch3() throws FileMatchingException {
        ExtensionMatcher matcher = new ExtensionMatcher("");

        // positives
        String[] names = new String[] {"", "txt.", "a_txt", "a.txt.", "test_files/a", "test_files/non-existing"};
        for (int i = 0; i < names.length; i++) {
            assertTrue("file name should be matched, name: " + names[i], matcher.match(names[i]));
            assertTrue("file name should be matched, name: " + names[i], matcher.match(new File(names[i])));
        }

        // negatives
        names = new String[] {"p.txt", "foo.bar.1.2-3.txt", ".txt", "test_files/a/p.txt", "a.tx",
            "test_files/non-existing.txt"};
        for (int i = 0; i < names.length; i++) {
            assertFalse("file name should not be matched, name: " + names[i], matcher.match(names[i]));
            assertFalse("file name should not be matched, name: " + names[i], matcher.match(new File(names[i])));
        }
    }

    /**
     * Tests match methods with valid arguments with matcher with two extensions. Filenames that match any
     * extension should be matched.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatch4() throws FileMatchingException {
        ExtensionMatcher matcher = new ExtensionMatcher(Arrays.asList(new String[] {"", "txt"}));

        // positives
        String[] names = new String[] {"", "txt.", "a_txt", "a.txt.", "test_files/a", "test_files/non-existing",
            "test_files/non-existing.txt", "p.txt", "foo.bar.1.2-3.txt", ".txt", "test_files/a/p.txt"};
        for (int i = 0; i < names.length; i++) {
            assertTrue("file name should be matched, name: " + names[i], matcher.match(names[i]));
            assertTrue("file name should be matched, name: " + names[i], matcher.match(new File(names[i])));
        }

        // negatives
        names = new String[] {"a.tx", "foo.bar", "a.txt.2"};
        for (int i = 0; i < names.length; i++) {
            assertFalse("file name should not be matched, name: " + names[i], matcher.match(names[i]));
            assertFalse("file name should not be matched, name: " + names[i], matcher.match(new File(names[i])));
        }
    }

}
